==> SomatiCA_1.0.0.tar.gz: SomatiCA/R/larsCBSsegment.R <==
larsCBSsegment <-
function(data, selection = .selection.default(), collapse.k = 0, ncores = 1, verbose = TRUE, variation.control = TRUE, rss = FALSE, S = 0.1, k = 50,...)
{
### Written by Mengjie Chen, Nov 2012 ####
### Modified in Jan 2012 ###
### Segmentation based on CBS followed by a model selection procedure ###
### Supporting multithread computing by package multicore ###	
	if(class(data) != "GRanges")
		stop("Wrong input format!")
	data_copy <- as.data.frame(data, stringsAsFactors=FALSE)	
  data_copy[, c("seqnames", "zygosity")] <- apply(data_copy[, c("seqnames", "zygosity")], 2, as.character)
  if(!any(grep("het", data_copy$zygosity)))
    stop("There is no heterozyguous site.\n")
	y <- data_copy[grep("het", data_copy$zygosity), ]
  newy <- SomatiCAFormat(y, missing=FALSE, verbose=verbose)
	suppressPackageStartupMessages(require(foreach))
	suppressPackageStartupMessages(require(DNAcopy))
	chrs <- unique(y$seqnames)
  i <- NULL
	if(ncores==1){  
		res <- foreach(i=1:length(chrs), .combine=rbind) %do% {		
			sub <- y[y$seqnames==chrs[i], ]
			if(verbose==TRUE){
				message(paste("Processing ", chrs[i], "...", sep=""))
			}
			smos <- denoise(data = sub$LAF, k = k)
			olarsCBS <- .larsCBS(z = smos, selection = selection, collapse.k = collapse.k, variation.control = variation.control, rss= rss, S = S, ...)
			if(olarsCBS$n.psi==0){
				allone <- c(as.character(chrs[i]), 0, sub$pos[length(smos)], olarsCBS$medians)
			  names(allone)[1:4] <- c("seqnames", "start", "end", "medLAF")
			}
			else{
				one <- cbind(as.character(sub$seqnames[olarsCBS$psiLARS]), sub$start[olarsCBS$psiLARS])
				one <- rbind(c(as.character(sub$seqnames[olarsCBS$psiLARS[1]]), 0), one)
				one<- cbind(one, c(one[-1, 2], sub$start[length(smos)]))
				allone <- cbind(one, olarsCBS$medians)
				names(allone)[1:4] <- c("seqnames", "start", "end", "medLAF")
			}	
			return(allone)
		}
	}
	if(ncores!=1){
	  if(.Platform$OS.type!= "unix"){
	    stop("Parallel computing is not supported on windows. Please set ncores = 1 .")
	  }
	  suppressPackageStartupMessages(require(doParallel))
	  registerDoParallel(ncores)
	  message(paste("Parallel computing on ", ncores, " cores\t", sep=""))
	  res <- foreach(i=1:length(chrs), .combine=rbind) %dopar% {  	
	    sub <- y[y$seqnames==chrs[i], ]
	    if(verbose==TRUE){
	      message(paste("Processing ", chrs[i], "...", sep=""))
	    }
	    smos <- denoise(data = sub$LAF, k = k)
	    olarsCBS <- .larsCBS(z = smos, selection = selection, collapse.k = collapse.k, variation.control = variation.control, rss= rss, S = S, ...)
	    if(olarsCBS$n.psi==0){
	      allone <- c(as.character(chrs[i]), 0, sub$pos[length(smos)], olarsCBS$medians)
	      names(allone)[1:4] <- c("seqnames", "start", "end", "medLAF")
      }
	    else{
	      one <- cbind(as.character(sub$seqnames[olarsCBS$psiLARS]), sub$start[olarsCBS$psiLARS])
	      one <- rbind(c(as.character(sub$seqnames[olarsCBS$psiLARS[1]]), 0), one)
	      one<- cbind(one, c(one[-1, 2], sub$start[length(smos)]))
	      allone <- cbind(one, olarsCBS$medians)
	      names(allone)[1:4] <- c("seqnames", "start", "end", "medLAF")
	     }	
	    return(allone)
	  }
	}
  colnames(res) <- c("seqnames", "start", "end", "medLAF")
  rownames(res) <- NULL
  res <- data.frame(res, stringsAsFactors = FALSE)
	results <- foreach(i=1:length(res$seqnames), .combine=c) %do% { 	
			sub <- elementMetadata(newy)$germLAF[start(newy) <  as.numeric(res$end[i]) & 
			                                     start(newy) >= as.numeric(res$start[i]) & 
                                           as.character(seqnames(newy)) == res$seqnames[i]]	
			return(round(median(sub, na.rm=TRUE), 4))
	}
  segment <- GRanges(seqnames=res$seqnames, 
                     ranges=IRanges(start=as.numeric(res$start), end=as.numeric(res$end)),
                     medLAF=as.numeric(res$medLAF),
                     medgLAF=results)  
	final <- list("segment" = segment, "hetsites" = newy)
	return(final)
}

==> SomatiCA_1.6.0.tar.gz: SomatiCA/R/larsCBSsegment.R <==
larsCBSsegment <-
function(data, selection = .selection.default(), collapse.k = 0, ncores = 1, verbose = TRUE, variation.control = TRUE, rss = FALSE, S = 0.1, k = 50,...)
{
### Written by Mengjie Chen, Nov 2012 ####
### Modified in Jan 2012 ###
### Segmentation based on CBS followed by a model selection procedure ###
### Supporting multithread computing by package multicore ###	
	if(class(data) != "GRanges")
		stop("Wrong input format!")
	data_copy <- as.data.frame(data, stringsAsFactors=FALSE)	
  data_copy[, c("seqnames", "zygosity")] <- apply(data_copy[, c("seqnames", "zygosity")], 2, as.character)
  if(!any(grep("het", data_copy$zygosity)))
    stop("There is no heterozyguous site.\n")
	y <- data_copy[grep("het", data_copy$zygosity), ]
  newy <- SomatiCAFormat(y, missing=FALSE, verbose=verbose)
	suppressPackageStartupMessages(require(foreach))
	suppressPackageStartupMessages(require(DNAcopy))
	chrs <- unique(y$seqnames)
  i <- NULL
	if(ncores==1){  
		res <- foreach(i=1:length(chrs), .combine=rbind) %do% {		
			sub <- y[y$seqnames==chrs[i], ]
			if(verbose==TRUE){
				message(paste("Processing ", chrs[i], "...", sep=""))
			}
			smos <- denoise(data = sub$LAF, k = k)
			olarsCBS <- .larsCBS(z = smos, selection = selection, collapse.k = collapse.k, variation.control = variation.control, rss= rss, S = S)# , ...)
			if(olarsCBS$n.psi==0){
				allone <- c(as.character(chrs[i]), 0, sub$start[length(smos)], olarsCBS$medians)
			  names(allone)[1:4] <- c("seqnames", "start", "end", "medLAF")
			}
			else{
				one <- cbind(as.character(sub$seqnames[olarsCBS$psiLARS]), sub$start[olarsCBS$psiLARS])
				one <- rbind(c(as.character(sub$seqnames[olarsCBS$psiLARS[1]]), 0), one)
				one<- cbind(one, c(one[-1, 2], sub$start[length(smos)]))
				allone <- cbind(one, olarsCBS$medians)
				names(allone)[1:4] <- c("seqnames", "start", "end", "medLAF")
			}	
			return(allone)
		}
	}
	if(ncores!=1){
	  if(.Platform$OS.type!= "unix"){
	    stop("Parallel computing is not supported on windows. Please set ncores = 1 .")
	  }
	  suppressPackageStartupMessages(require(doParallel))
	  registerDoParallel(ncores)
	  message(paste("Parallel computing on ", ncores, " cores\t", sep=""))
	  res <- foreach(i=1:length(chrs), .combine=rbind) %dopar% {  	
	    sub <- y[y$seqnames==chrs[i], ]
	    if(verbose==TRUE){
	      message(paste("Processing ", chrs[i], "...", sep=""))
	    }
	    smos <- denoise(data = sub$LAF, k = k)
	    olarsCBS <- .larsCBS(z = smos, selection = selection, collapse.k = collapse.k, variation.control = variation.control, rss= rss, S = S, ...)
	    if(olarsCBS$n.psi==0){
	      allone <- c(as.character(chrs[i]), 0, sub$start[length(smos)], olarsCBS$medians)
	      names(allone)[1:4] <- c("seqnames", "start", "end", "medLAF")
	    }
	    else{
	      one <- cbind(as.character(sub$seqnames[olarsCBS$psiLARS]), sub$start[olarsCBS$psiLARS])
	      one <- rbind(c(as.character(sub$seqnames[olarsCBS$psiLARS[1]]), 0), one)
	      one<- cbind(one, c(one[-1, 2], sub$start[length(smos)]))
	      allone <- cbind(one, olarsCBS$medians)
	      names(allone)[1:4] <- c("seqnames", "start", "end", "medLAF")
	     }	
	    return(allone)
	  }
	}
  if(!is.vector(res)){
    colnames(res) <- c("seqnames", "start", "end", "medLAF")
    rownames(res) <- NULL
    res <- data.frame(res, stringsAsFactors = FALSE)
	  results <- foreach(i=1:length(res$seqnames), .combine=c) %do% { 	
		  	sub <- elementMetadata(newy)$germLAF[start(newy) <  as.numeric(res$end[i]) & 
			                                     start(newy) >= as.numeric(res$start[i]) & 
                                           as.character(seqnames(newy)) == res$seqnames[i]]	
			  return(round(median(sub, na.rm=TRUE), 4))
	  }
    segment <- GRanges(seqnames=res$seqnames, 
                     ranges=IRanges(start=as.numeric(res$start), end=as.numeric(res$end)),
                     medLAF=as.numeric(res$medLAF),
                     medgLAF=results) 
  }
  else{
    segment <- GRanges(seqnames=res[1], 
                       ranges=IRanges(start=as.numeric(res[2]), end=as.numeric(res[3])),
                       medLAF=as.numeric(res[4]),
                       medgLAF=median(newy$germLAF)) 
  }
	final <- list("segment" = segment, "hetsites" = newy)
	return(final)
}
